/*
 * cdata.h
 * Copyright (C) Raul Massano Brás 2012 <rbras@iseg.utl.pt>
 *
 * MulTyLink is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MulTyLink is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef CDATA_H
#define CDATA_H

#include <QObject>
#include "cproblem.h"



class CData : public QObject
{
private:
    int maxId;
    site_info ***mat; //mat(i,j) stores a pointer so a site_info
    size_t m,n;  //Dimensions of mat
    size_t numTypes; //Number of protected area types
    size_t numSites;
    size_t zoom;
    std::string *names; //names of the protected areas types
    std::vector<int> *pa_ids; //pa_ids(i) is a vector with all the pa ids of pa type i
    std::vector<site_info *> sites; //Save information about all the sites
    std::vector<site_info *> fake_sites; //Save information about the PA
    std::map<int,int> fake_ids; //For each PA assicates the PA id to a fake id bigger then maxId
    vdMap_t vdMap; //Associates to each id a pointer with the vertex data

    double maxFriction;
    double *adjDist;
    int inputType; //Type of the input file

    MainWindow *mainFrm;
public:
    CData(MainWindow *mf);
    ~CData();

    bool readFile(const QString& file);
    QPoint getGrid() const {return QPoint(m,n);}
    size_t getNumTypes() const {return numTypes;}
    bool buildGraph(CGraph_u **gk,double hf);
    size_t getNumPa(size_t i) const {return pa_ids[i].size();} //Return the number of protected areas of type i
    const site_info *getData(int i, int j) const {if(i<0||i>=(int)m||j<0||j>=(int)n) return NULL; return mat[i][j];}
    const std::vector<int> *getPaIds() const {return pa_ids;}

    const std::string& getPAName(int i) const {return names[i];}
    size_t getZoom() {return zoom;}
    double getMaxFriction()const {return maxFriction;}

    int getMaxId() const {return maxId;}
    size_t getNumSites() const {return sites.size();}
    const site_info *getSiteData(size_t i) const {return sites[i];}

    const std::vector<site_info *>& getFakeSites() const {return fake_sites;}

    bool isBarrier(int id, size_t j) { return vdMap[id]->barriers[j];}
    double getCost(int id)  {return vdMap[id]->cost;}
    bool isTerm(int id) {if(vdMap[id]->siteType==0) return false; else return true;}
    int getSiteType(int id) {return vdMap[id]->siteType;}
    vertex_data *getVertexDataById(int id) {return vdMap[id];}

private:
    bool insertEdge(int ori,int dest,CGraph_u *g) const;
    double calcDist(vertex_data *v1,vertex_data *v2);

};

#endif // CDATA_H
