/*
 * cproblem.cpp
 * Copyright (C) Raul Massano Brás 2012 <rbras@iseg.utl.pt>
 *
 * MulTyLink is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MulTyLink is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "stringutils.h"
#include "cscene.h"
#include "cproblem.h"
#include <algorithm>
#include <fstream>
#include <QThread>
#include "cdata.h"

using namespace std;

int binSearch(const vector<int> x, int val)
{
  //Because we keep adjList sorted by id we do a binary search here
  int left=0,right=(int)x.size()-1;
  if(right<0) return -1;
  int p=(right+left)/2;
  for(;;) {
    if(right-left<8) { //do a linear search
      for(int i=left;i<=right;i++) if(x[i]==val) return i;
      return -1;
    }
    if(x[p]==val) return p;
    else if(x[p]<val) left=p+1;
    else right=p-1;
    p=(right+left)/2;
  }

}


CProblem::CProblem(MainWindow *mf)
{

    //names=NULL;
    //reserve=NULL;
    //pa_ids=NULL;
    data=NULL;
    mainFrm=mf;

    gk=NULL;
    pars.limit=100;
    int r=QThread::idealThreadCount();
    pars.nt=(r>0?r:1);
    pars.epsilon=1e-14;

    pars.numRep=100;
    pars.time_limit=60*60;

    //numSites=0;
    numTypes=0;
    zoom=1;
}

CProblem::~CProblem()
{
    //if(names) delete [] names;
    //if(pa_ids) delete [] pa_ids;
    /*
    if(reserve) {
        for(size_t i=0;i<numTypes;i++) {
            delete [] reserve[i];
        }
        delete [] reserve;
    }
    */
    if(gk) {
        for(size_t i=0;i<numTypes;++i) {
            if(gk[i]) delete gk[i];
        }
        delete [] gk;
    }
    if(data) delete data; //delete the data, including the user data in the graph
}



bool CProblem::readData(const QString& file)
{
    //Read file with site info and adjacencies

    data=new CData(mainFrm);
    if(!data->readFile(file)) {
        delete data;
        data=NULL;
        return false;
    }

    numTypes=data->getNumTypes();
    pars.limit=data->getMaxFriction();
    gk=new CGraph_u * [numTypes];
    for(size_t i=0;i<numTypes;++i) gk[i]=new CGraph_u;

    if(!data->buildGraph(gk,pars.limit)) {
        delete data;
        data=NULL;
        for(size_t i=0;i<numTypes;i++) delete gk[i];
        delete gk; gk=NULL;
        return false;
    }
    //g->printGraph();

    grid=data->getGrid();
    if(numTypes<=4) {
        pars.nperm=1;
        for(size_t i=2;i<=numTypes;i++) pars.nperm*=i;
    }
    else pars.nperm=100;
    //numSites=g->size();
    zoom=data->getZoom();

    return true;
}

size_t CProblem::getNumPa(size_t i) const {
    return data->getNumPa(i);
}

const site_info *CProblem::getData(int i, int j) const
{
    return data->getData(i,j);
}

void CProblem::setParams(double friction_limit, size_t nt,size_t np,int nr, int tl)
{
    pars.nt=nt; pars.nperm=np; pars.numRep=nr; pars.time_limit=tl;
    if(friction_limit!=pars.limit) {
        pars.limit=friction_limit;
        if(gk) {
            for(size_t i=0;i<data->getNumTypes();++i) {
                if(gk[i]) delete gk[i];
            }
            delete gk;
        }
        gk=new CGraph_u *[data->getNumTypes()];
        for(size_t i=0;i<data->getNumTypes();++i) gk[i]=new CGraph_u;

        data->buildGraph(gk,friction_limit);
    }
}
