/*
 * cscene.h
 * Copyright (C) Raul Massano Brás 2012 <rbras@iseg.utl.pt>
 *
 * MulTyLink is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MulTyLink is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef CSCENE_H
#define CSCENE_H

#include <QtGui/QMainWindow>
#include <QCloseEvent>
#include <QActionGroup>
#include <QProgressBar>
#include <QLabel>
#include <QTime>
#include "cgraph.h"
#include "popupmessage.h"

struct problemSol;
class CProblem;
class CSolveThread;

typedef enum {MSG_PROGRESS=1,MSG_FINISHED=1<<1,MSG_LOG=1<<2,MSG_LABEL=1<<3} THREAD_MSG;

enum {COLOR_MultiType=0,COLOR_Background,COLOR_Barriers,COLOR_Solution,COLOR_NoType,COLOR_Friction,COLOR_END};

struct logMsg_s
{
    QString msg;
};
Q_DECLARE_METATYPE(logMsg_s);


namespace Ui {
    class CScene;
}

class CScene : public QMainWindow {
    Q_OBJECT
    Q_DISABLE_COPY(CScene)
public:
    explicit CScene(QWidget *parent,CProblem *);
    virtual ~CScene();

protected:
    virtual void changeEvent(QEvent *e);
    virtual void closeEvent(QCloseEvent *event);

private:
    Ui::CScene *m_ui;
    QWidget *mainWin;
    CProblem *problem;
    bool newSol;
    problemSol *solProb;
    QProgressBar *pb;
    QTime time;
    bool *checkedTypes,*checkedTypesBarr;
    bool typeChanged,colorChanged;
    bool firstView;

    static QColor baseColors[COLOR_END];
    QColor *typeColor;

    //CSolveThread *thread;
    QThread *thread;

    QAction **act,**actBarr;
    QPixmap *bmap,*bmapc;
    QLabel *label;
    QLabel *clabel;
    PopupMessage *popup;
    void cleanSol();
    void enableMenus(bool);

    void DrawImage(QPainter *dc);



public:
    void setProgress(int x) {pb->setValue(x); update();}
    void setZoomValue(int val);

private slots:
    void on_actionAllBarr_triggered();
    void on_actionAll_triggered();
    void on_actionSave_Image_triggered();
    void on_actionSolve_2_triggered();
    void on_actionSet_Params_triggered();
    void on_actionSave_as_triggered();
    void on_actionClose_triggered();
    void threadMessages(int , uint, logMsg_s);
    void threadProgressMessages(int);

    void on_horizontalSlider_sliderMoved(int position);

    void on_actionSolve_using_Greedy_triggered();

    void on_actionColors_triggered();

    void on_actionSave_colour_scheme_triggered();

    void on_actionLoad_colour_scheme_triggered();

protected:
    void paintEvent(QPaintEvent *);
    void mousePressEvent(QMouseEvent *ev);
    void mouseReleaseEvent(QMouseEvent *ev);

signals:
    void winCloseSignal(CScene *);
};

#endif // CSCENE_H
