/*
 * cscene.cpp
 * Copyright (C) Raul Massano Brás 2012 <rbras@iseg.utl.pt>
 *
 * MulTyLink is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MulTyLink is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "csetcolours.h"
#include "ui_csetcolours.h"
#include "cscene.h"
#include <QColorDialog>
#include <QMessageBox>

csetcolours::csetcolours(QWidget *parent,size_t nc,QColor *bc, QColor *tc,CData *data) :
    QDialog(parent),
    ui(new Ui::csetcolours)
{
    numTypes=nc;
    baseColors=bc;
    typeColors=tc;
    ui->setupUi(this);

    tmpTypeColors=new QColor [numTypes];
    for(size_t i=0;i<COLOR_END;i++) tmpBaseColors[i]=baseColors[i];
    for(size_t i=0;i<numTypes;i++) tmpTypeColors[i]=typeColors[i];

    setButtonColor(ui->bBackground,baseColors[COLOR_Background]);
    setButtonColor(ui->bSol,baseColors[COLOR_Solution]);
    setButtonColor(ui->bMultiType,baseColors[COLOR_MultiType]);
    setButtonColor(ui->bNoType,baseColors[COLOR_NoType]);
    setButtonColor(ui->bBarriers,baseColors[COLOR_Barriers]);
    setButtonColor(ui->bFriction,baseColors[COLOR_Friction]);

    hz=new QHBoxLayout* [numTypes];
    lb=new QLabel* [numTypes];
    pb=new QPushButton* [numTypes];

    for(size_t i=0;i<numTypes;++i) {
        hz[i] = new QHBoxLayout();
        hz[i]->setObjectName(QString::fromUtf8("hz1%1").arg(i+1));
        lb[i] = new QLabel(this);
        lb[i]->setObjectName(QString::fromUtf8("lb%1").arg(i+1));
        lb[i]->setText(QString(data->getPAName(i).c_str()));

        hz[i]->addWidget(lb[i]);

        pb[i] = new QPushButton(this);
        pb[i]->setObjectName(QString::fromUtf8("bType%1").arg(i+1));

        hz[i]->addWidget(pb[i]);


        ui->verticalLayout->addLayout(hz[i]);
        QObject::connect( pb[i], SIGNAL(clicked()),this, SLOT( bTypeClicked() ) );
        bMap[pb[i]]=i;
        setButtonColor(pb[i],typeColors[i]);
    }

    bChanged=false;
}

csetcolours::~csetcolours()
{
    delete [] hz;
    delete [] pb;
    delete [] lb;
    delete ui;
}

void csetcolours::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void csetcolours::on_bSol_clicked()
{
    QColorDialog dlg(this);
    if(dlg.exec()==QColorDialog::Accepted) {
        tmpBaseColors[COLOR_Solution]=dlg.currentColor();
        setButtonColor(ui->bSol,dlg.currentColor());
    }
}

void csetcolours::on_bMultiType_clicked()
{
    QColorDialog dlg(this);
    if(dlg.exec()==QColorDialog::Accepted) {
        tmpBaseColors[COLOR_MultiType]=dlg.currentColor();
        setButtonColor(ui->bMultiType,dlg.currentColor());
    }

}

void csetcolours::on_bBackground_clicked()
{
    QColorDialog dlg(this);
    if(dlg.exec()==QColorDialog::Accepted) {
        tmpBaseColors[COLOR_Background]=dlg.currentColor();
        setButtonColor(ui->bBackground,dlg.currentColor());
    }

}

void csetcolours::on_bBarriers_clicked()
{
    QColorDialog dlg(this);
    if(dlg.exec()==QColorDialog::Accepted) {
        tmpBaseColors[COLOR_Barriers]=dlg.currentColor();
        setButtonColor(ui->bBarriers,dlg.currentColor());
    }

}

void csetcolours::on_bNoType_clicked()
{
    QColorDialog dlg(this);
    if(dlg.exec()==QColorDialog::Accepted) {
        tmpBaseColors[COLOR_NoType]=dlg.currentColor();
        setButtonColor(ui->bNoType,dlg.currentColor());
    }

}
void csetcolours::on_bFriction_clicked()
{
    QColorDialog dlg(this);
    if(dlg.exec()==QColorDialog::Accepted) {
        tmpBaseColors[COLOR_Friction]=dlg.currentColor();
        setButtonColor(ui->bFriction,dlg.currentColor());
    }

}

void csetcolours::bTypeClicked()
{
    QPushButton *button = (QPushButton *)sender();
    QColorDialog dlg(this);
    if(dlg.exec()==QColorDialog::Accepted) {
        tmpTypeColors[bMap[button]]=dlg.currentColor();
        setButtonColor(button,dlg.currentColor());
    }

}

void csetcolours::setButtonColor(QPushButton *button,const QColor& color)
{
    /*
    QPalette palette;
    QBrush brush(color);
    brush.setStyle(Qt::SolidPattern);
    palette.setBrush(QPalette::Active, QPalette::Button, brush);
    palette.setBrush(QPalette::Inactive, QPalette::Button, brush);
    palette.setBrush(QPalette::Disabled, QPalette::Button, brush);
    button->setPalette(palette);
    */
    QString cl=tr("rgb(%1,%2,%3,%4)").arg(color.red()).arg(color.green()).arg(color.blue()).arg(color.alpha());
    QString style;
    style=tr("background:%1;color:%1; font-size:12px;font-weight:bold; border-radius:10%;margin:1px; border:3px solid black")
                .arg(cl);

    button->setStyleSheet(style);


    bChanged=true;
}

void csetcolours::on_buttonBox_accepted()
{
    for(size_t i=0;i<COLOR_END;i++) baseColors[i]=tmpBaseColors[i];
    for(size_t i=0;i<numTypes;i++) typeColors[i]=tmpTypeColors[i];
    accept();
}

void csetcolours::on_buttonBox_rejected()
{
    bChanged=false;
    reject();
}

