/*
 * csetparams.cpp
 * Copyright (C) Raul Massano Brás 2012 <rbras@iseg.utl.pt>
 *
 * MulTyLink is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MulTyLink is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "csetparams.h"
#include "ui_csetparams.h"
#include <QMessageBox>
#include <QTableWidgetItem>
#include "limits.h"
CSetParams::CSetParams(QWidget *parent, const params *pars,const size_t& numTypes, const double& max_friction) :
    QDialog(parent),
    m_ui(new Ui::CSetParams)
{
    m_ui->setupUi(this);
    m_ui->lineEdit_limit->setText(QString("%1").arg(pars->limit));
    m_ui->lineEdit_nt->setText(QString("%1").arg(pars->nt));
    m_ui->lineEdit_nPerm->setText(QString("%1").arg(pars->nperm));
    m_ui->label_maxFriction->setText(QString("%1").arg(max_friction));
    m_ui->lineEdit_numRuns->setText(QString("%1").arg(pars->numRep));
    int tl=pars->time_limit;
    if(tl==INT_MAX) tl=0;
    else tl/=60;
    m_ui->lineEdit_timeLimit->setText(QString("%1").arg(tl));

    if(numTypes<=4) {
        maxPerm=1;
        for(size_t i=2;i<=numTypes;i++) maxPerm*=i;
    }
    else maxPerm=100;
}

CSetParams::~CSetParams()
{
    delete m_ui;
}

void CSetParams::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void CSetParams::getVals(double& friction_limit,size_t& nt,size_t& np,int& numRuns, int& timeLimit)
{
    //lambda=m_ui->lineEdit_lambda->text().toDouble();
    //alpha=m_ui->lineEdit_alpha->text().toDouble();
    friction_limit=m_ui->lineEdit_limit->text().toDouble();
    nt=m_ui->lineEdit_nt->text().toInt();
    np=m_ui->lineEdit_nPerm->text().toInt();
    numRuns=m_ui->lineEdit_numRuns->text().toInt();
    timeLimit=m_ui->lineEdit_timeLimit->text().toInt();
    if(timeLimit==0) timeLimit=INT_MAX;
    else timeLimit*=60;
}

void CSetParams::onOK()
{
    double x;
    bool ok;
    x=m_ui->lineEdit_limit->text().toDouble(&ok);
    if(!ok || x<0) {
        QMessageBox::critical(this,tr("Error"),tr("Values for friction threshold must be >=0."));
        return;
    }
    x=m_ui->lineEdit_nt->text().toInt(&ok);
    if(!ok || x<1 || x>64) {
        QMessageBox::critical(this,tr("Error"),tr("Values for max. threads must be between 0 and 64."));
        return;
    }
    x=m_ui->lineEdit_nPerm->text().toInt(&ok);
    if(!ok || x<1 || x>maxPerm) {
        QMessageBox::critical(this,tr("Error"),tr("Value for permutations must be between 1 and %2.").arg(maxPerm));
        return;
    }
    x=m_ui->lineEdit_timeLimit->text().toInt(&ok);
    if(!ok || x<0) {
        QMessageBox::critical(this,tr("Error"),tr("Values for time limit must be nonnegative integers."));
        return;
    }
    x=m_ui->lineEdit_numRuns->text().toInt(&ok);
    if(!ok || x<=0) {
        QMessageBox::critical(this,tr("Error"),tr("Values for number of runs must be postive integers."));
        return;
    }
    accept();

}
