/*
 * cthreads.h
 * Copyright (C) Raul Massano Brás 2012 <rbras@iseg.utl.pt>
 *
 * MulTyLink is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MulTyLink is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _CTHREADS_H_
#define _CTHREADS_H_

#include <QThread>
#include <QWaitCondition>
#include <QMutex>
#include <set>
#include "mainwindow.h"


//Identifiers for the messages sent to main window


enum {THREAD_RUN1=1,THREAD_RUN2,THREAD_ABORT};

struct my_msgbuf {
  long mtype;
  struct {
    void *arg1;
    void *arg2;
  } args;
};

struct graphSol_threads {
  set<vDesc_u> vertices;
  double cost;
  CGraph_u **gsol; //Save the best graph for each type
};


class MainWindow;
class CConnComp;

using namespace std;

class CWorkingThread : public QThread
{
    Q_OBJECT
private:
    CGraph_u **gk;
    CProblem *problem;
    vector<vDesc_d> *terminals;
    vector<vDesc_d> *rep;
    set< pair<vDesc_d,vDesc_d> > *term_par;
    int begin,end;
public:
    explicit CWorkingThread(CGraph_u **ggk,CProblem *p,vector<vDesc_u> *t,vector<vDesc_u> *r,
                   set< pair<vDesc_u,vDesc_u> > *tp,int b, int e):
                    gk(ggk),problem(p),terminals(t),rep(r),term_par(tp),begin(b),end(e) {}
    ~CWorkingThread() {}

private:
    CGraph_d *buildDirectedGraphOfType(size_t k,set<size_t>& vertInSol);
    CGraph_d *buildDirectedGraphOfTypeFromSol(size_t k);
    void cleanCyclesFromType(set<vDesc_u>& sol,size_t type);
    CGraph_u **makeMinimal(graphSol_threads *currSol, set<vDesc_u>& sol);
    bool removeVertex(CGraph_u *gk,vDesc_u idx,size_t k,vector< set<vDesc_u> >& comps,vector<vDesc_u>& adjs);
    bool fviol(const set<vDesc_u>& conn,size_t k);

protected:
    void run();
signals:
    void threadProgressSignal(int);
    void threadMsgSignal(int, uint, logMsg_s );
};



class CSolveThread : public QThread
{

    Q_OBJECT

private:
    CProblem *problem;
    CScene *scene;
    problemSol *resSol;
    CGraph_u **gk;
    CWorkingThread **thread;
    size_t nt;
    size_t K;
    //vector for the terminals
    vector<vDesc_u> *terminals;
    //For each type save in rep a representative of the terminals. It can be more then one
    //if the graph is not connected and for each type we have several terminal sets
    vector<vDesc_u> *rep;
    //set with pairs of terminals for each type
    set< pair<vDesc_u,vDesc_u> > *term_par;

    bool verifyGraph();
    int randInt(int a, int b) {return qrand() % ((b + 1) - a) + a;}
    void shuffle(int n, vector<int>& v);

public:
    explicit CSolveThread(CScene *f,CProblem *p, problemSol *ps):QThread(f),problem(p),scene(f),resSol(ps),
            gk(p->getGraphTypePtr())
    {
        nt=problem->getParams()->nt;
        K=problem->getNumTypes();

        term_par=new set< pair<vDesc_u,vDesc_u> >[K];
        rep=new vector<vDesc_u>[K];
        terminals=new vector<vDesc_u>[K];
        thread=new CWorkingThread* [nt];
        for(uint i=0;i<nt;i++) thread[i]=NULL;

    }
    ~CSolveThread() {

        //clean memory
        delete [] thread;
        delete [] terminals;
        delete [] term_par;
        delete [] rep;

    }


protected:
    void run();

signals:
    void threadMsgSignal(int, uint, logMsg_s);
    void threadProgressSignal(int);
};





#endif
