/*
 * cscene.cpp
 * Copyright (C) Raul Massano Brás 2012 <rbras@iseg.utl.pt>
 *
 * MulTyLink is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MulTyLink is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CTHREADSGREEDY_H
#define CTHREADSGREEDY_H

#include <QThread>
#include "mainwindow.h"
#include "cproblem.h"
#include "cdata.h"

class CThreadsGreedy : public QThread
{
    Q_OBJECT
private:
    CProblem *problem;
    CScene *scene;
    problemSol *resSol;
    CGraph_u **gk_u;
    const params *pars;
    CGraph_d *gk;
    int nt;
    int K;
    CData *data;

    vector< vector<int> > *terms_in_comp;
    set< pair<int,int> > *term_par;
    vector<int> *terminals;

public:
    explicit CThreadsGreedy(CScene *f,CProblem *p,problemSol *ps):QThread(f),problem(p),scene(f),resSol(ps),
            gk_u(p->getGraphTypePtr()),pars(problem->getParams())
    {
        nt=problem->getParams()->nt;
        K=problem->getNumTypes();

        data=problem->getDataPtr();

        terms_in_comp = new vector< vector<int> > [K];
        term_par = new set< pair<int,int> > [K];
        terminals=new vector<int>[K];

        //Build the directed graphs and fill the terminals vector
        gk=new CGraph_d [K];
        //Vertices
        for(int k=0;k<K;k++) {
            vIter_u vi,vi_end;
            for(tie(vi,vi_end)=gk_u[k]->getVertices();vi!=vi_end;++vi) {
                int id=gk_u[k]->getId(*vi);
                gk[k].insertVertex(id);
                if(data->getSiteType(id)==k+1) terminals[k].push_back(id);
            }
            sort(terminals[k].begin(),terminals[k].end());
        }

        //Edges
        CData *data=problem->getDataPtr();
        for(int k=0;k<K;k++) {
            eIter_u ei,ei_end;
            for(tie(ei,ei_end)=gk_u[k]->getEdges();ei!=ei_end;++ei){
                vDesc_u ori=gk_u[k]->getEdgeSource(*ei);
                vDesc_u dest=gk_u[k]->getEdgeTarget(*ei);
                int idOri=gk_u[k]->getId(ori);
                int idDest=gk_u[k]->getId(dest);
                vDesc_d ori_d=gk[k].findVertex(idOri).first;
                vDesc_d dest_d=gk[k].findVertex(idDest).first;
                double c;
                //c=(data->isTerm(idDest)?0:data->getCost(idDest)+pars->epsilon);
                c=(data->getSiteType(idDest)==k+1?0:data->getCost(idDest)+pars->epsilon);
                gk[k].insertEdge(ori_d,dest_d,c);
                //c=(data->isTerm(idOri)?0:data->getCost(idOri)+pars->epsilon);
                c=(data->getSiteType(idOri)==k+1?0:data->getCost(idOri)+pars->epsilon);
                gk[k].insertEdge(dest_d,ori_d,c);
            }
        }
    }
    ~CThreadsGreedy()
    {
        delete [] terms_in_comp;
        delete [] term_par;
        delete [] terminals;
        delete [] gk;
    }
private:
    void initTermPar();
    void calcSolution(set<int>& sol);
    int randInt(int a, int b) {return qrand() % ((b + 1) - a) + a;}
    void makeMinimal(set<int>& sol);
    CGraph_u *createGraphsFromSolution(set<int>&sol);
    bool removeVertex(CGraph_u& ngk,vDesc_u idx,int k,vector<vDesc_u>& adjs);
    bool fviol(const set<int>& conn,int k);
    void calcSolutionByType(const set<int>& sol);

protected:
    void run();
signals:
    void threadProgressSignal(int);
    void threadMsgSignal(int, uint, logMsg_s );

public slots:

};

#endif // CTHREADSGREEDY_H
