/*
 * mainwindow.cpp
 * Copyright (C) Raul Massano Brás 2012 <rbras@iseg.utl.pt>
 *
 * MulTyLink is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MulTyLink is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "version.h"
#include <QMessageBox>
#include <QFileDialog>
#include "cabout.h"
#include "cscene.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindowClass)
{
    ui->setupUi(this);

    //this is to use '.' as the decimal point
    setlocale(LC_NUMERIC,"en_US");

    problem=NULL;
    qRegisterMetaType<logMsg_s>();
    log=new LogDlg(this);

    move(0,0);
    log->move(width()+20,0);


    log->show();
    log->insertText(tr("Multiple Type Linkages\n"));
    log->insertText(tr("MulTyLink version %1.%2.%3\n").arg(VER_MAJOR).arg(VER_MINOR).arg(VER_RELEASE));

    //Initialize random number generator
    QTime time = QTime::currentTime();
    qsrand((uint)time.msec());
}

MainWindow::~MainWindow()
{
    delete ui;
    if(problem) delete problem;

    //Do not delete scene. It is deleted in childClosed
}

void MainWindow::on_actionExit_triggered()
{
    close();
}

void MainWindow::on_actionOpen_triggered()
{

    //get the file name
    QString file=QFileDialog::getOpenFileName(this,tr("Select file"), ".", tr("MulTyLink Files (*.txt)"));
    if(!file.size()) return;

    //Create the problem
    problem=new CProblem(this);
    //Read the data
    if(!problem->readData(file)) {
        delete problem;
        problem=NULL;
        return;
    }

    //Create the window to display the data
    CScene *scene=new CScene(this,problem);

    scene->move(0,0);
    scene->show();
    scene->update();


    //There is no need to save the window pointer since we will be notified when
    //it closes and the pointer passed to childClosed so we can delete it

    connect(scene,SIGNAL(winCloseSignal(CScene * )),SLOT(childClosed(CScene *)),Qt::QueuedConnection);

    //Disable open menu

    ui->actionOpen->setEnabled(false);

}


void MainWindow::childClosed(CScene *win)
{

    delete win;
    if(problem) {
        delete problem;
        problem=NULL;
    }
    ui->actionOpen->setEnabled(true);
}

void MainWindow::on_actionAbout_triggered()
{
    /*
    QMessageBox::about(this,tr("About"),trUtf8("\nCorrodors\nVersion: %1.%2.%3.%4\nAuthor: Raul Brás\nLicense: GPL V3\n\n©2009-2010").
                       arg(VER_MAJOR).arg(VER_MINOR).arg(VER_RELEASE).arg(VER_BUILD));
    */
    CAbout dlg;
    dlg.exec();
}

void MainWindow::on_actionAbout_Qt_triggered()
{
    QMessageBox::aboutQt(this);
}
